# include "macros.h"
# include "utils.h"
# include "imageFile-io.h"
# include "faceImageIO.h"

int readFaceHeader(FILE *fpIn, float *facePos)
{
  char comment[MAXLINE_PGM], *pc;
  int foundCoords, headerState;

  foundCoords = FALSE;
  headerState = 0;
  while(!foundCoords) {  /* Seek next comment */

    if (readPnmCommentLine(fpIn, comment, &headerState) <= 0) {
      fprintf(stderr, "readFaceHeader: Unable to find faceCoord comment\n");
      return(-1);
    }
    /* Check comment for face coords*/
    if ((pc = strstr(comment, "facecoords")) != NULL) {
      pc += strlen("facecoords");
      if (sscanf(pc, "%f %f %f %f %f %f", 
		 facePos, facePos+1, facePos+2, facePos+3, 
		 facePos+4, facePos+5) <0) {
	fprintf(stderr,"readFaceHeader: Cannot read face coords string ", pc);
	return(-1);
      }
      foundCoords = TRUE;
    }
          
  } 
  return(0);
}

void save_faceImage_wcomment(char *path, char *comment,
			     float *imageFlt, int nx, int ny, 
			     char *markX, float* facePos)
{
  char dataOut[MAXLEN], header[MAXLEN];
  int  sizeOutput, nHead, count;
  unsigned char *image;
  int k, length, width;
  int xcent[2];
  FILE *outfile; 

  length = 3;
  width = 0;

  sprintf(header, "P5\n#%s\n%d %d\n%d\n", comment, nx, ny, 255);

  /* Open image */
  strcpy(dataOut,path);
  strcat(dataOut, ".pgm");
  if ( (outfile = fopen(dataOut, "wb")) == NULL ) 
    error("Can't open output file", dataOut);

  /* Save header */
  sizeOutput = strlen(header) * sizeof( char );
  if (!fwrite(header, sizeOutput, 1, outfile))
    error("Unable to write to file", dataOut);

  /* Quantize float image */
  quantize_float_image(&image, imageFlt, nx, ny);

  if (markX[0] == 'y') {
    /* mark crosses at face positons */
    for(k=0; k<3; k++) {
      xcent[0] = ROUND(facePos[2*k]);
      xcent[1] = ROUND(facePos[2*k+1]);
      drawCross(image, nx, ny, xcent, length, width, 255);
    }
  }

  /* Save quantized image */
  sizeOutput = nx * ny * sizeof( char );
  if (!fwrite((char *) image, sizeOutput, 1, outfile))
    error("Unable to write to file", dataOut);

  fclose(outfile);
  free(image);
}

void drawCross(unsigned char *image, int nx, int ny, 
	       int *pos, int length, int width, 
	       unsigned char gray)
{
  int ix,iy;
  for(ix=pos[0]-length; ix<=pos[0]+length; ix++) 
    if (ix>=0 && ix<nx)
      for(iy=pos[1]-width; iy<=pos[1]+width; iy++) 
	if (iy>=0 && iy<ny)
	  image[index(ix,iy,nx)] = gray;
  for(iy=pos[1]-length; iy<=pos[1]+length; iy++) 
    if (iy>=0 && iy<ny)
      for(ix=pos[0]-width; ix<=pos[0]+width; ix++) 
	if (ix>=0 && ix<nx)
	  image[index(ix,iy,nx)] = gray;
}
