/********************************************************
 scaleFace: Version 1.0: Oct., 1998 

 To compile (on SGI's): 
  make scaleFace
********************************************************/

# include "macros.h"
# include "utils.h"
# include "endianness.h"
# include "imageFile-io.h"
# include "mapping.h"
# include "faceImageIO.h"

unsigned char grey = 127;
int cropBorder = 30;
float eyeDist0 = 25.0;  /* rescale original images to have inter-eye
                          distance = 25 pixels. (lin interp used)
                          (SHOULD FIX THIS) */

main(argc, argv)
 int argc;
 char **argv;
 {
 
  float *imageFlt, *warped, *cropImage;
  float x_orig[2], warpParam[8], invWarp[8], centPt[2];
  float eyeDist, scl, theta, x_old[2];
  float facePos[6];
 
  int cropBox[4];
  int pgmInput, nHead, nxImage, nyImage;
  int nxWarp, nyWarp;
  int nxCrop, nyCrop;
  int i,k, ix, iy;
  int nTrain, kTrain;
  unsigned char *image, *cimage[3];

  char fnInput[255], fnTrain[MAXLEN][MAXLEN],scrName[MAXLEN];
  char rootName[MAXLEN];
  char fnOutput[255], pathNameOutput[MAXLEN], outputFileRoot[MAXLEN];
  char pathNameInput[MAXLEN];
  char comment[MAXLEN];
  char markX[MAXLEN];
  char inImageType[3];

  FILE *fpIn;

  /******************************/
  /* Read input data from stdin */
  /******************************/
  fprintf(stderr, "Inter-eye distance : ");
  scanf("%e", &eyeDist0);

  fprintf(stderr, "Output file path and rootname: \n");
  if (read_file_path_and_root(pathNameOutput, outputFileRoot) == EOF)
   exit(0);
  fprintf(stderr, "Mark X's in output pgm face images? (y/n): \n");
  scanf("%s", &markX);

  /*****************************/
  /* Read training image names */
  /*****************************/
  fprintf(stderr, "Training image file path: \n");
  scanf("%s", pathNameInput);
  fprintf(stderr, "Input a list of training image names (terminate with \"null\"): ");
  nTrain =0;
  while(scanf("%s", fnTrain[nTrain]) != EOF) {
   if (checkForNull(fnTrain[nTrain]) == 0) 
    break;
   nTrain++;
   if (nTrain >= MAXLEN)
    break;
  }
  
  fpIn = NULL;

  for(kTrain=0; kTrain<nTrain; kTrain++) {
    /*******************/
    /* Read image data */
    /*******************/
    strcpy(fnInput, pathNameInput);
    strcat(fnInput, fnTrain[kTrain]);

    /* Open image file */
    if ((fpIn = freopenInputFile(fnInput, fpIn)) == NULL) {
      fprintf(stderr, "Can't open image file %s\n",fnInput);
      exit(0);
    }
    if (stripPnmHeader(fpIn, &nxImage, &nyImage, inImageType) < 0) {
      fprintf(stderr," Image file: %s, type: %s\n", fnInput, inImageType);
      error("Unrecognized image file type",""); 
    }
    rewind(fpIn);
    /* Read face coords and file type */
    if (0> readFaceHeader(fpIn, facePos))
       fprintf(stderr, " cannot read face coords for %s\n", fnInput);
    rewind(fpIn);

    if (inImageType[0]=='P') {
     if (inImageType[1] =='6') {
      read_ppm_image(cimage, fpIn, fnInput, &nxImage, &nyImage);
      image = cimage[0];
      free(cimage[1]);
      free(cimage[2]);
     } else if (inImageType[1] =='5') {
      read_pgm_image(&image, fpIn, fnInput, &nxImage, &nyImage);
     } else
      error("Unrecognized input file type ", inImageType);

     grabFloatMemory(&imageFlt, nxImage*nyImage, "imageFlt");
     for(i=0; i<nxImage*nyImage; i++)
       imageFlt[i] = (float) image[i];
     free(image);
    } else if (inImageType[0]=='F') {
     if (inImageType[1] =='5') {
      read_pfm_image(&imageFlt, fpIn, fnInput, &nxImage, &nyImage);
     } else
      error("Unrecognized input file type ", inImageType);
    } else
     error("Unrecognized input file type ", inImageType);


    /* Inter eye distance */
    eyeDist = (facePos[0] - facePos[2]) * (facePos[0] - facePos[2]);
    eyeDist += (facePos[1] - facePos[3]) * (facePos[1] - facePos[3]);
    eyeDist = sqrt(eyeDist);

    /* Rescale face image */
    for(i=0; i<8;i++)
      warpParam[i] = 0.0;
    theta = atan2(facePos[3]-facePos[1], facePos[2] - facePos[0]);
    scl = eyeDist0/eyeDist;
    warpParam[2] = scl * cos(theta) ;
    warpParam[3] = -scl * sin(theta) ;
    warpParam[4] = -warpParam[3];
    warpParam[5] = warpParam[2];
    x_orig[0] = x_orig[1] = 0.0;
    
    warpAndBound(imageFlt, nxImage, nyImage, 
                   &warped, &nxWarp, &nyWarp,
                   x_orig, warpParam, grey, 1, invWarp);
    fprintf(stderr, "scl = %f, input image size %d %d, output size %d %d\n",
                 scl, nxImage, nyImage, nxWarp, nyWarp);
    free(imageFlt);
    imageFlt = warped;
    nxImage = nxWarp;
    nyImage = nyWarp;

    /* Get warped face positions */
    if (-1 == invertRational(invWarp, warpParam))
      error("Rational transform for prewarp nearly singular","");
    for(k=0; k<3; k++) {
       x_old[0] = facePos[2*k];
       x_old[1] = facePos[2*k+1];
       rationalPtSample(x_old, x_orig, warpParam, centPt); 
       facePos[2*k] = centPt[0];
       facePos[2*k+1] = centPt[1];
    }

    /* Find cropBox from warped face positions */
    cropBox[0] = cropBox[2] = floor(facePos[0]);
    cropBox[1] = cropBox[3] = ceil(facePos[1]);
    for(k=1; k<3; k++) {
       cropBox[0] = min(floor(facePos[2*k]), cropBox[0]);
       cropBox[1] = min(floor(facePos[2*k+1]), cropBox[1]);
       cropBox[2] = max(ceil(facePos[2*k]), cropBox[2]);
       cropBox[3] = max(ceil(facePos[2*k+1]), cropBox[3]);
    }

    cropBox[0] -= cropBorder;
    cropBox[0] = max(0, cropBox[0]);
    cropBox[1] -= cropBorder;
    cropBox[1] = max(0, cropBox[1]);
    cropBox[2] += cropBorder;
    cropBox[2] = min(nxImage-1, cropBox[2]);
    cropBox[3] += cropBorder;
    cropBox[3] = min(nyImage-1, cropBox[3]);

    nxCrop = cropBox[2] - cropBox[0] + 1;
    nyCrop = cropBox[3] - cropBox[1] + 1;
    grabFloatMemory(&cropImage, nxCrop*nyCrop, "cropImage");
    /* Crop float image ... pad with grey */
    for(i=0; i<nxCrop*nyCrop; i++)
       cropImage[i] = (float) grey;
    for(iy=0; iy<nyCrop; iy++)
     for(ix=0; ix<nxCrop; ix++) 
       if ((ix+cropBox[0] >= 0) && (ix+cropBox[0] < nxImage) 
            && (iy+cropBox[1] >= 0) && (iy+cropBox[1] < nyImage))
         cropImage[index(ix,iy,nxCrop)] =
          imageFlt[index(ix+cropBox[0], iy+cropBox[1], nxImage)];
      
    /* Shift face params to crop box coords */
    for(k=0; k<3; k++) {
      facePos[2*k] -= cropBox[0];
      facePos[2*k+1] -= cropBox[1];
    }

    /* Build float image output file name */
    strcpy(rootName,outputFileRoot);
    strcat(rootName,fnTrain[kTrain]);
    rootName[strlen(rootName)-4] = '\0';  /* prune .ppm extension */

    /* Build header comment */
    sprintf(comment,"# facecoords %.1f %.1f %.1f %.1f %.1f %.1f\n", 
        facePos[0], facePos[1],
        facePos[2], facePos[3],
        facePos[4], facePos[5]);

    strcpy(fnOutput, pathNameOutput);
    strcat(fnOutput, "pfm/");
    strcat(fnOutput, rootName);
    save_pfm_image_wcomment(fnOutput, comment, cropImage, nxCrop, nyCrop);

    strcpy(fnOutput, pathNameOutput);
    strcat(fnOutput, "pgm/");
    strcat(fnOutput, rootName);
    save_faceImage_wcomment(fnOutput, comment,
                            cropImage, nxCrop, nyCrop, markX, facePos);
   }
   fclose(fpIn);
 }
