
#include "macros.h"
#include "linearAlg.h"

float innerProd(float *a, float *b, int n)
 {
   int i;
   float val = 0.0;
   for(i=0; i<n; i++)
    val += a[i] * b[i];
   return(val);
 }

float norm(float *x, int n)
 {
  int i;
  float nrm;

  nrm = 0.0;
  for(i=0; i<n; i++)
   nrm += x[i] * x[i];
  nrm = sqrt( nrm );
  return(nrm);
 }

double normDbl(double *x, int n)
 {
  int i;
  double nrmDbl = 0.0e+00;
  for(i=0; i<n; i++)
   nrmDbl += x[i] * x[i];
  nrmDbl = sqrt( nrmDbl );
  return(nrmDbl);
 }

float rescale(float *x, int n)
{
  int i;
  float nrm;

  nrm = norm(x, n);
  if ( nrm > 0.0 )
   for(i=0; i<n; i++)
    x[i] /= nrm;
  else
   for(i=0; i<n; i++)
    x[i] = 0.0;

  return nrm;
 }

/* x = projection of x onto s perp (norm s = 1) */
void project(float *x, float *s, int n )
 {
  float inner;
  int i;

  inner = innerProd( x, s, n );

  for(i=0; i<n; i++)
   x[i] -= inner * s[i];
 }

/* y = projection of x onto s perp (norm s = 1) */
void projectUnit(float *y, float *x, float *s, int n)
 {
  float inner;
  int i;

  inner = innerProd( x, s, n );

  for(i=0; i<n; i++)
   y[i] = x[i] - inner * s[i];
 }

/* c <- aXb */
void crossProd(float a[3],float b[3],float c[3])

 {
    c[0] = a[1] * b[2] - a[2]* b[1];
    c[1] = a[2] * b[0] - a[0]* b[2];
    c[2] = a[0] * b[1] - a[1]* b[0];
 }

